@@ -1,6 +1,6 @@
 /*
     This file is part of the KContacts framework.
-    Copyright (c) 2015 Laurent Montel <montel@kde.org>
+    Copyright (c) 2015-2016 Laurent Montel <montel@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -109,7 +109,7 @@ void RelatedTest::shouldParseRelated()
                          "VERSION:4.0\n"
                          "N:LastName;FirstName;;;\n"
                          "UID:c80cf296-0825-4eb0-ab16-1fac1d522a33@xxxxxx.xx\n"
-                         "RELATED:friend"
+                         "RELATED:friend\n"
                          "REV:2015-03-14T09:24:45+00:00\n"
                          "FN:FirstName LastName\n"
                          "END:VCARD\n");
@@ -118,8 +118,32 @@ void RelatedTest::shouldParseRelated()
     const KContacts::AddresseeList lst = vcard.parseVCards(vcarddata);
     QCOMPARE(lst.count(), 1);
     QCOMPARE(lst.at(0).relationShips().count(), 1);
+    const KContacts::Related related = lst.at(0).relationShips().at(0);
+    QCOMPARE(related.related(), QStringLiteral("friend"));
+
 }
 
+void RelatedTest::shouldParseRelatedWithArgument()
+{
+    QByteArray vcarddata("BEGIN:VCARD\n"
+                         "VERSION:4.0\n"
+                         "N:LastName;FirstName;;;\n"
+                         "UID:c80cf296-0825-4eb0-ab16-1fac1d522a33@xxxxxx.xx\n"
+                         "RELATED;FOO1=bla1,blo1;FOO2=bla2,blo2:friend\n"
+                         "REV:2015-03-14T09:24:45+00:00\n"
+                         "FN:FirstName LastName\n"
+                         "END:VCARD\n");
+
+    KContacts::VCardTool vcard;
+    const KContacts::AddresseeList lst = vcard.parseVCards(vcarddata);
+    QCOMPARE(lst.count(), 1);
+    QCOMPARE(lst.at(0).relationShips().count(), 1);
+    const KContacts::Related related = lst.at(0).relationShips().at(0);
+    QCOMPARE(related.related(), QStringLiteral("friend"));
+    QCOMPARE(related.parameters().count(), 2);
+}
+
+
 void RelatedTest::shouldParseWithoutRelated()
 {
     QByteArray vcarddata("BEGIN:VCARD\n"
@@ -235,6 +259,7 @@ void RelatedTest::shouldNotExportInVcard3()
                         "VERSION:3.0\r\n"
                         "EMAIL:foo@kde.org\r\n"
                         "N:;;;;\r\n"
+                        "RELATED;FOO1=bla1,blo1;FOO2=bla2,blo2:friend\r\n"
                         "UID:testuid\r\n"
                         "END:VCARD\r\n\r\n");
     QCOMPARE(ba, expected);
