@@ -44,6 +44,7 @@
 #include "related.h"
 #include "addresseelist.h"
 #include "resourcelocatorurl.h"
+#include "fieldgroup.h"
 
 namespace KContacts
 {
@@ -572,6 +573,7 @@ public:
       Set homepage.
      */
     void setUrl(const ResourceLocatorUrl &url);
+    //kf6: remove it
     void setUrl(const QUrl &url);
 
     /**
@@ -733,6 +735,8 @@ public:
      */
     PhoneNumber::List phoneNumbers() const;
 
+    void setPhoneNumbers(const PhoneNumber::List &phoneNumbers);
+
     /**
       Return list of phone numbers with a special type.
 
@@ -1035,6 +1039,11 @@ public:
     void setImppList(const Impp::List &imppList);
     void insertImpp(const Impp &impp);
 
+    //FieldGroup
+    FieldGroup::List fieldGroupList() const;
+    void setFieldGroupList(const FieldGroup::List &fieldGroupList);
+    void insertFieldGroup(const FieldGroup &fieldGroup);
+
 private:
     class Private;
     QSharedDataPointer<Private> d;
