@@ -60,7 +60,9 @@ public:
         Car = 1024,   /**< Car phone */
         Isdn = 2048,  /**< ISDN connection */
         Pcs = 4096,   /**< Personal Communication Service*/
-        Pager = 8192  /**< Pager */
+        Pager = 8192,  /**< Pager */
+        //TODO add Text support vcard4
+        Undefined = 16384 /** Undefined number type */
     };
 
     Q_DECLARE_FLAGS(Type, TypeFlag)
@@ -224,7 +226,7 @@ KCONTACTS_EXPORT QDataStream &operator<<(QDataStream &stream, const PhoneNumber
  * @param stream The stream to read from
  * @param number The phone number object to deserialize into
  */
-KCONTACTS_EXPORT QDataStream &operator>>(QDataStream &stream, const PhoneNumber &number);
+KCONTACTS_EXPORT QDataStream &operator>>(QDataStream &stream, PhoneNumber &number);
 
 }
 Q_DECLARE_TYPEINFO(KContacts::PhoneNumber, Q_MOVABLE_TYPE);
