@@ -1,7 +1,7 @@
 /*
     This file is part of the KContacts framework.
     Copyright (c) 2003 Tobias Koenig <tokoe@kde.org>
-    Copyright (c) 2015 Laurent Montel <montel@kde.org>
+    Copyright (c) 2015-2016 Laurent Montel <montel@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -27,6 +27,8 @@
 #include "lang.h"
 #include "gender.h"
 #include "related.h"
+#include "fieldgroup.h"
+#include "kcontacts_debug.h"
 #include <QtCore/QString>
 #include <QtCore/QBuffer>
 #include <QDebug>
@@ -92,8 +94,14 @@ void VCardTool::addParameter(VCardLine &line, VCard::Version version, const QStr
         Q_FOREACH (const QString &valueStr, valueStringList) {
             line.addParameter(valueStr, QString());
         }
+    } else if (version == VCard::v3_0) {
+        line.addParameter(key, valueStringList.join(QLatin1Char(',')));
     } else {
-        line.addParameter(key, valueStringList.join(QStringLiteral(",")));
+        if (valueStringList.count() < 2) {
+            line.addParameter(key, valueStringList.join(QLatin1Char(',')));
+        } else {
+            line.addParameter(key, QLatin1Char('"') + valueStringList.join(QLatin1Char(',')) + QLatin1Char('"'));
+        }
     }
 }
 
@@ -150,17 +158,12 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             address.append((*it).country().replace(QLatin1Char(';'),
                                                    QLatin1String("\\;")));
 
-            VCardLine adrLine(QStringLiteral("ADR"), address.join(QStringLiteral(";")));
-            if (version == VCard::v2_1 && needsEncoding(address.join(QStringLiteral(";")))) {
+            VCardLine adrLine(QStringLiteral("ADR"), address.join(QLatin1Char(';')));
+            if (version == VCard::v2_1 && needsEncoding(address.join(QLatin1Char(';')))) {
                 adrLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
                 adrLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
             }
 
-            VCardLine labelLine(QStringLiteral("LABEL"), (*it).label());
-            if (version == VCard::v2_1 && needsEncoding((*it).label())) {
-                labelLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
-                labelLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
-            }
 
             const bool hasLabel = !(*it).label().isEmpty();
             QMap<QString, Address::TypeFlag>::ConstIterator typeIt;
@@ -176,14 +179,33 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                 }
             }
 
+            if (hasLabel) {
+                if (version == VCard::v4_0) {
+                    if (!(*it).label().isEmpty()) {
+                        adrLine.addParameter(QStringLiteral("LABEL"), QStringLiteral("\"%1\"").arg((*it).label()));
+                    }
+                } else {
+                    VCardLine labelLine(QStringLiteral("LABEL"), (*it).label());
+                    if (version == VCard::v2_1 && needsEncoding((*it).label())) {
+                        labelLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
+                        labelLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
+                    }
+                    addParameter(labelLine, version, QStringLiteral("TYPE"), labelLineType);
+                    card.addLine(labelLine);
+                }
+            }
+            if (version == VCard::v4_0) {
+                Geo geo = (*it).geo();
+                if (geo.isValid()) {
+                    QString str;
+                    str.sprintf("\"geo:%.6f,%.6f\"", geo.latitude(), geo.longitude());
+                    adrLine.addParameter(QStringLiteral("GEO"), str);
+                }
+            }
             if (!isEmpty) {
                 addParameter(adrLine, version, QStringLiteral("TYPE"), addreLineType);
                 card.addLine(adrLine);
             }
-            if (hasLabel) {
-                addParameter(labelLine, version, QStringLiteral("TYPE"), labelLineType);
-                card.addLine(labelLine);
-            }
         }
 
         // BDAY
@@ -201,7 +223,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                     (*catIt).replace(QLatin1Char(','), QLatin1String("\\,"));
                 }
 
-                VCardLine catLine(QStringLiteral("CATEGORIES"), categories.join(QStringLiteral(",")));
+                VCardLine catLine(QStringLiteral("CATEGORIES"), categories.join(QLatin1Char(',')));
                 card.addLine(catLine);
             }
         }
@@ -221,19 +243,17 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             card.addLine(line);
         }
 
-        if (version == VCard::v4_0) {
-            const Related::List relatedList = (*addrIt).relationShips();
-            Related::List::ConstIterator relatedIt;
-            Related::List::ConstIterator relatedEnd(relatedList.end());
-            for (relatedIt = relatedList.begin(); relatedIt != relatedEnd; ++relatedIt) {
-                VCardLine line(QStringLiteral("RELATED"), (*relatedIt).related());
-                QMapIterator<QString, QStringList> i((*relatedIt).parameters());
-                while (i.hasNext()) {
-                    i.next();
-                    line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
-                }
-                card.addLine(line);
+        const Related::List relatedList = (*addrIt).relationShips();
+        Related::List::ConstIterator relatedIt;
+        Related::List::ConstIterator relatedEnd(relatedList.end());
+        for (relatedIt = relatedList.begin(); relatedIt != relatedEnd; ++relatedIt) {
+            VCardLine line(QStringLiteral("RELATED"), (*relatedIt).related());
+            QMapIterator<QString, QStringList> i((*relatedIt).parameters());
+            while (i.hasNext()) {
+                i.next();
+                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
             }
+            card.addLine(line);
         }
         // CLASS only for version == 3.0
         if (version == VCard::v3_0) {
@@ -249,7 +269,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                 QMapIterator<QString, QStringList> i((*langIt).parameters());
                 while (i.hasNext()) {
                     i.next();
-                    line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                    line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
                 }
                 card.addLine(line);
             }
@@ -278,10 +298,10 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                             line.addParameter(QStringLiteral("PREF"), QString());
                         }
                     } else {
-                        line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                        line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
                     }
                 } else {
-                    line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                    line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
                 }
             }
             card.addLine(line);
@@ -348,8 +368,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
         name.append((*addrIt).suffix().replace(QLatin1Char(';'),
                                                QLatin1String("\\;")));
 
-        VCardLine nLine(QStringLiteral("N"), name.join(QStringLiteral(";")));
-        if (version == VCard::v2_1 && needsEncoding(name.join(QStringLiteral(";")))) {
+        VCardLine nLine(QStringLiteral("N"), name.join(QLatin1Char(';')));
+        if (version == VCard::v2_1 && needsEncoding(name.join(QLatin1Char(';')))) {
             nLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
             nLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
         }
@@ -390,8 +410,8 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             organization.append((*addrIt).department().replace(QLatin1Char(';'),
                                 QLatin1String("\\;")));
         }
-        VCardLine orgLine(QStringLiteral("ORG"), organization.join(QStringLiteral(";")));
-        if (version == VCard::v2_1 && needsEncoding(organization.join(QStringLiteral(";")))) {
+        VCardLine orgLine(QStringLiteral("ORG"), organization.join(QLatin1Char(';')));
+        if (version == VCard::v2_1 && needsEncoding(organization.join(QLatin1Char(';')))) {
             orgLine.addParameter(QStringLiteral("charset"), QStringLiteral("UTF-8"));
             orgLine.addParameter(QStringLiteral("encoding"), QStringLiteral("QUOTED-PRINTABLE"));
         }
@@ -442,8 +462,11 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             QStringList lst;
             for (typeIt = mPhoneTypeMap.constBegin(); typeIt != typeEnd; ++typeIt) {
                 if (typeIt.value() & (*phoneIt).type()) {
-                    lst << typeIt.key();
-
+                    if (version == VCard::v4_0) {
+                        lst << typeIt.key().toLower();
+                    } else {
+                        lst << typeIt.key();
+                    }
                 }
             }
             if (!lst.isEmpty()) {
@@ -487,7 +510,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             QMapIterator<QString, QStringList> i(url.parameters());
             while (i.hasNext()) {
                 i.next();
-                line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
             }
             card.addLine(line);
         }
@@ -536,13 +559,24 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                         QMapIterator<QString, QStringList> i(url.parameters());
                         while (i.hasNext()) {
                             i.next();
-                            line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                            line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
                         }
                         card.addLine(line);
                     }
                 }
             }
         }
+        //FieldGroup
+        Q_FOREACH (const FieldGroup &group, (*addrIt).fieldGroupList()) {
+            VCardLine line(group.fieldGroupName(), group.value());
+            QMapIterator<QString, QStringList> i(group.parameters());
+            while (i.hasNext()) {
+                i.next();
+                line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
+            }
+            card.addLine(line);
+        }
+
         // IMPP (supported in vcard 3 too)
         Q_FOREACH (const Impp &impp, (*addrIt).imppList()) {
             VCardLine line(QStringLiteral("IMPP"), impp.address());
@@ -551,7 +585,7 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
             while (i.hasNext()) {
                 i.next();
                 if (i.key().toLower() != QStringLiteral("x-service-type")) {
-                    line.addParameter(i.key(), i.value().join(QStringLiteral(",")));
+                    line.addParameter(i.key(), i.value().join(QLatin1Char(',')));
                 }
             }
             card.addLine(line);
@@ -603,7 +637,15 @@ QByteArray VCardTool::createVCards(const Addressee::List &list,
                     const QDate date = QDate::fromString(value, Qt::ISODate);
                     QDateTime dt = QDateTime(date);
                     dt.setTime(QTime());
-                    card.addLine(VCardLine(QStringLiteral("ANNIVERSARY"), createDateTime(dt, version)));
+                    VCardLine line(QStringLiteral("ANNIVERSARY"), createDateTime(dt, version));
+                    card.addLine(line);
+                }
+            } else if (identifier.toLower() == QLatin1String("x-kaddressbook-x-spousesname") && version == VCard::v4_0) {
+                if (!value.isEmpty()) {
+                    VCardLine line(QStringLiteral("RELATED"), QStringLiteral(";"));
+                    line.addParameter(QStringLiteral("TYPE"), QStringLiteral("spouse"));
+                    line.addParameter(QStringLiteral("VALUE"), value);
+                    card.addLine(line);
                 }
             } else {
                 VCardLine line(identifier, value);
@@ -626,6 +668,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
     static const QLatin1Char semicolonSep(';');
     static const QLatin1Char commaSep(',');
     QString identifier;
+    QString group;
 
     Addressee::List addrList;
     const VCard::List vCardList = VCardParser::parseVCards(vcard);
@@ -645,8 +688,15 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
             // iterate over the lines
             for (lineIt = lines.begin(); lineIt != lines.end(); ++lineIt) {
                 identifier = (*lineIt).identifier().toLower();
+                group = (*lineIt).group();
+                if (!group.isEmpty()) {
+                    KContacts::FieldGroup groupField(group + QLatin1Char('.') + (*lineIt).identifier());
+                    groupField.setParameters((*lineIt).parameterMap());
+                    groupField.setValue((*lineIt).value().toString());
+                    addr.insertFieldGroup(groupField);
+                }
                 // ADR
-                if (identifier == QLatin1String("adr")) {
+                else if (identifier == QLatin1String("adr")) {
                     Address address;
                     const QStringList addrParts = splitString(semicolonSep, (*lineIt).value().toString());
                     if (addrParts.count() > 0) {
@@ -680,8 +730,26 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                     }
 
                     address.setType(type);
-                    if (!(*lineIt).parameter(QStringLiteral("label")).isEmpty()) {
-                        address.setLabel((*lineIt).parameter(QStringLiteral("label")));
+                    QString label = (*lineIt).parameter(QStringLiteral("label"));
+                    if (!label.isEmpty()) {
+                        if (label.length() > 1) {
+                            if (label.at(0) == QLatin1Char('"') && label.at(label.length() - 1) == QLatin1Char('"')) {
+                                label = label.mid(1, label.length() - 2 );
+                            }
+                        }
+                        address.setLabel(label);
+                    }
+                    QString geoStr = (*lineIt).parameter(QStringLiteral("geo"));
+                    if (!geoStr.isEmpty()) {
+                        geoStr.remove(QLatin1Char('\"'));
+                        geoStr.remove(QStringLiteral("geo:"));
+                        if (geoStr.contains(QLatin1Char(','))) {
+                            QStringList arguments = geoStr.split(QLatin1Char(','));
+                            KContacts::Geo geo;
+                            geo.setLatitude(arguments.at(0).toDouble());
+                            geo.setLongitude(arguments.at(1).toDouble());
+                            address.setGeo(geo);
+                        }
                     }
                     addr.insertAddress(address);
                 }
@@ -732,40 +800,18 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 else if (identifier == QLatin1String("impp")) {
                     QString imppStr = (*lineIt).value().toString();
                     Impp impp;
-                    impp.setAddress(imppStr);
                     impp.setParameters((*lineIt).parameterMap());
                     if (!(*lineIt).parameter(QStringLiteral("x-service-type")).isEmpty()) {
+                        impp.setAddress(imppStr);
                         const QString serviceType = (*lineIt).parameter(QStringLiteral("x-service-type")).toLower();
-                        if (serviceType == QLatin1String("facebook")) {
-                            impp.setType(KContacts::Impp::Facebook);
-                        } else if (serviceType == QLatin1String("jabber")) {
-                            impp.setType(KContacts::Impp::Jabber);
-                        } else if (serviceType == QLatin1String("sip")) {
-                            impp.setType(KContacts::Impp::Sip);
-                        } else if (serviceType == QLatin1String("aim")) {
-                            impp.setType(KContacts::Impp::Aim);
-                        } else if (serviceType == QLatin1String("msn")) {
-                            impp.setType(KContacts::Impp::Msn);
-                        } else if (serviceType == QLatin1String("twitter")) {
-                            impp.setType(KContacts::Impp::Twitter);
-                        } else if (serviceType == QLatin1String("googletalk")) {
-                            impp.setType(KContacts::Impp::GoogleTalk);
-                        } else if (serviceType == QLatin1String("xmpp")) {
-                            impp.setType(KContacts::Impp::Xmpp);
-                        } else if (serviceType == QLatin1String("icq")) {
-                            impp.setType(KContacts::Impp::Icq);
-                        } else if (serviceType == QLatin1String("yahoo")) {
-                            impp.setType(KContacts::Impp::Yahoo);
-                        } else if (serviceType == QLatin1String("qq")) {
-                            impp.setType(KContacts::Impp::Qq);
-                        } else if (serviceType == QLatin1String("gadugadu")) {
-                            impp.setType(KContacts::Impp::GaduGadu);
-                        } else if (serviceType == QLatin1String("owncloud-handle")) {
-                            impp.setType(KContacts::Impp::Ownclound);
-                        } else if (serviceType == QLatin1String("skype")) {
-                            impp.setType(KContacts::Impp::Skype);
-                        } else {
-                            qDebug() << "unknown service type " << serviceType;
+                        imppService(serviceType, impp);
+                    } else {
+                        const int pos = imppStr.indexOf(QLatin1Char(':'));
+                        if (pos != -1) {
+                            const QString serviceType = imppStr.left(pos);
+                            const QString address = imppStr.right(imppStr.length() - pos - 1);
+                            impp.setAddress(address);
+                            imppService(serviceType, impp);
                         }
                     }
                     addr.insertImpp(impp);
@@ -927,10 +973,11 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
                 // ORGANIZATION
                 else if (identifier == QLatin1String("org")) {
                     const QStringList orgParts = splitString(semicolonSep, (*lineIt).value().toString());
-                    if (orgParts.count() > 0) {
+                    const int orgPartsCount(orgParts.count());
+                    if (orgPartsCount > 0) {
                         addr.setOrganization(orgParts.at(0));
                     }
-                    if (orgParts.count() > 1) {
+                    if (orgPartsCount > 1) {
                         addr.setDepartment(orgParts.at(1));
                     }
                     if (!(*lineIt).parameter(QStringLiteral("sort-as")).isEmpty()) {
@@ -980,14 +1027,19 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
                 // TEL
                 else if (identifier == QLatin1String("tel")) {
+                    //TODO implement load vcard4
                     PhoneNumber phone;
                     phone.setNumber((*lineIt).value().toString());
 
                     PhoneNumber::Type type;
                     bool foundType = false;
+#if 0
+                    qDebug() << " (*lineIt).parameters" << (*lineIt).parameterMap();
+                    qDebug() << " (*lineIt).value()" << (*lineIt).value().toString();
+#endif
                     const QStringList types = (*lineIt).parameters(QStringLiteral("type"));
-                    QStringList::ConstIterator typeEnd(types.end());
-                    for (QStringList::ConstIterator it = types.begin(); it != typeEnd; ++it) {
+                    QStringList::ConstIterator typeEnd(types.constEnd());
+                    for (QStringList::ConstIterator it = types.constBegin(); it != typeEnd; ++it) {
                         type |= mPhoneTypeMap[(*it).toUpper()];
                         foundType = true;
                     }
@@ -1003,6 +1055,7 @@ Addressee::List VCardTool::parseVCards(const QByteArray &vcard) const
 
                 // TZ
                 else if (identifier == QLatin1String("tz")) {
+                    //TODO add vcard4 support
                     TimeZone tz;
                     const QString date = (*lineIt).value().toString();
 
@@ -1307,6 +1360,16 @@ Key VCardTool::parseKey(const VCardLine &line) const
             key.setType(Key::Custom);
             key.setCustomTypeString(line.parameter(QStringLiteral("type")));
         }
+    } else if (params.contains(QStringLiteral("mediatype"))) {
+        const QString param = line.parameter(QStringLiteral("mediatype")).toLower();
+        if (param == QLatin1String("application/x-x509-ca-cert")) {
+            key.setType(Key::X509);
+        } else if (param == QLatin1String("application/pgp-keys")) {
+            key.setType(Key::PGP);
+        } else {
+            key.setType(Key::Custom);
+            key.setCustomTypeString(line.parameter(QStringLiteral("type")));
+        }
     }
 
     return key;
@@ -1329,12 +1392,22 @@ VCardLine VCardTool::createKey(const Key &key, VCard::Version version) const
         line.setValue(key.textData());
     }
 
-    if (key.type() == Key::X509) {
-        line.addParameter(QStringLiteral("type"), QStringLiteral("X509"));
-    } else if (key.type() == Key::PGP) {
-        line.addParameter(QStringLiteral("type"), QStringLiteral("PGP"));
-    } else if (key.type() == Key::Custom) {
-        line.addParameter(QStringLiteral("type"), key.customTypeString());
+    if (version == VCard::v4_0) {
+        if (key.type() == Key::X509) {
+            line.addParameter(QStringLiteral("MEDIATYPE"), QStringLiteral("application/x-x509-ca-cert"));
+        } else if (key.type() == Key::PGP) {
+            line.addParameter(QStringLiteral("MEDIATYPE"), QStringLiteral("application/pgp-keys"));
+        } else if (key.type() == Key::Custom) {
+            line.addParameter(QStringLiteral("MEDIATYPE"), key.customTypeString());
+        }
+    } else {
+        if (key.type() == Key::X509) {
+            line.addParameter(QStringLiteral("type"), QStringLiteral("X509"));
+        } else if (key.type() == Key::PGP) {
+            line.addParameter(QStringLiteral("type"), QStringLiteral("PGP"));
+        } else if (key.type() == Key::Custom) {
+            line.addParameter(QStringLiteral("type"), key.customTypeString());
+        }
     }
 
     return line;
@@ -1407,3 +1480,39 @@ QStringList VCardTool::splitString(QChar sep, const QString &str) const
 
     return list;
 }
+
+
+void VCardTool::imppService(const QString &serviceType, KContacts::Impp &impp) const
+{
+    if (serviceType == QLatin1String("facebook")) {
+        impp.setType(KContacts::Impp::Facebook);
+    } else if (serviceType == QLatin1String("jabber")) {
+        impp.setType(KContacts::Impp::Jabber);
+    } else if (serviceType == QLatin1String("sip")) {
+        impp.setType(KContacts::Impp::Sip);
+    } else if (serviceType == QLatin1String("aim")) {
+        impp.setType(KContacts::Impp::Aim);
+    } else if (serviceType == QLatin1String("msn")) {
+        impp.setType(KContacts::Impp::Msn);
+    } else if (serviceType == QLatin1String("twitter")) {
+        impp.setType(KContacts::Impp::Twitter);
+    } else if (serviceType == QLatin1String("googletalk")) {
+        impp.setType(KContacts::Impp::GoogleTalk);
+    } else if (serviceType == QLatin1String("xmpp")) {
+        impp.setType(KContacts::Impp::Xmpp);
+    } else if (serviceType == QLatin1String("icq")) {
+        impp.setType(KContacts::Impp::Icq);
+    } else if (serviceType == QLatin1String("yahoo")) {
+        impp.setType(KContacts::Impp::Yahoo);
+    } else if (serviceType == QLatin1String("qq")) {
+        impp.setType(KContacts::Impp::Qq);
+    } else if (serviceType == QLatin1String("gadugadu")) {
+        impp.setType(KContacts::Impp::GaduGadu);
+    } else if (serviceType == QLatin1String("owncloud-handle")) {
+        impp.setType(KContacts::Impp::Ownclound);
+    } else if (serviceType == QLatin1String("skype")) {
+        impp.setType(KContacts::Impp::Skype);
+    } else {
+        qCDebug(KCONTACTS_LOG) << "unknown service type " << serviceType;
+    }
+}
